import click
from transformers import AutoTokenizer, RobertaModel
import numpy as np
from data import Data, from_db, connect, data_dir
from tqdm import tqdm
import torch
from pathlib import Path

@click.command(name="word:max-sequence")
def max_sequence():
    """calculate the maximum token length given the story titles"""
    db = connect()
    longest = db.sql("""
        select
            title
        from stories
        order by length(title) desc
        limit 5000
    """).df()
    db.close()

    tokenizer = AutoTokenizer.from_pretrained("roberta-base")
    tokens = tokenizer(longest['title'].to_list())
    print(f"{max([len(x) for x in tokens['input_ids']])}")

@click.command(name="word:train")
def train():
    """TODO"""
    table = from_db(Data.Titles)
    n_classes = 10

@click.command(name="word:embed")
@click.option('-c', '--chunks', type=int, default=5000, show_default=True)
@click.option('--embedding_dest', help="path to save embeddings as np array", type=Path, default=Path(data_dir() / 'sequence_embeddings.npy'), show_default=True)
@click.option('--token_dest', help="path to save tokens as np array", type=Path, default=Path(data_dir() / 'sequence_tokens.npy'), show_default=True)
def embed(chunks, embedding_dest, token_dest):
    """ given titles, generate tokens and word embeddings and saves to disk """

    # init models
    device = torch.device('cuda:0')
    tokenizer = AutoTokenizer.from_pretrained("roberta-base")
    model = RobertaModel.from_pretrained("roberta-base")
    model.to(device)

    # load data
    db = connect()
    table = db.sql("""
        select
        title
        from stories
        order by id desc
    """).df()
    db.close()

    # normalize text
    table['title'] = table['title'].str.normalize('NFKD').str.encode('ascii', errors='ignore').str.decode('utf-8')

    # generate embeddings from list of titles
    def get_embeddings(titles):
        # create tokens, padding to max width 
        tokens = tokenizer(titles, add_special_tokens = True, truncation = True, padding = "max_length", max_length=92, return_attention_mask = True, return_tensors = "pt")
        tokens = tokens.to(device)
        with torch.no_grad():
            outputs = model(**tokens)
        return tokens.to(torch.device('cpu')), outputs.last_hidden_state.to(torch.device('cpu'))

    tokens = []
    embeddings = []
    chunks = np.array_split(table['title'].to_numpy(), chunks)
    chunk_iter = tqdm(chunks, 'embedding')
    for chunk in chunk_iter:
        data = chunk.tolist()
        token, embedding = get_embeddings(data)
        arr = embedding.detach().numpy()
        embeddings.append(arr)
        tokens.append(token)

    embeddings = np.concatenate(embeddings)
    tokens = np.concatenate(tokens)
    np.save(embedding_dest, embeddings)
    np.save(token_dest, tokens)

@click.command(name="word:distance")
def distance():
    """TODO: measure distance between sequence embeddings"""
    distances = distance.cdist(classes, classes, 'euclidean')
    np.fill_diagonal(distances, np.inf)
    min_index = (np.argmin(distances))
    closest = np.unravel_index(min_index, distances.shape)
    distances.flatten().shape
